" MIT License. Copyright (c) 2017 Carl Smedstad
" Author:       Carl Smedstad
" Last Change:  March 29, 2017
" URL:          https://github.com/carlsmedstad/vim-sourcer

function! sourcer#source_files(...)
  " Source the files supplied as arguments if they have the ".vim" extension.
  " Unlets "g:{plugin}_loaded" variable to properly source loaded plugins if
  " "g:sourcer_source_loaded=1".
  " Returns: the number of sourced files.
  let g:sourcer#nbr#loaded#sourced = 0
  let g:sourcer#verbose#output = ""
  let l:nbr_of_sourced = 0
  let l:files = copy(a:000)
  if empty(l:files) && getbufvar(expand("%"), "&ft") == "vim"
    let l:nbr_of_sourced += s:source_file(expand("%"))
  else
    for vimfile in filter(l:files, 'v:val =~# "\\.vim$"')
      let l:nbr_of_sourced += s:source_file(vimfile)
    endfor
  endif
  return l:nbr_of_sourced
endfunction

function! sourcer#source_bufs()
  " Sources all buffers containing vimscript files.
  " Returns: the number of sourced files.
  let g:sourcer#nbr#loaded#sourced = 0
  let g:sourcer#verbose#output = ""
  let l:nbr_of_sourced = 0
  let l:buflist = filter(range(1, bufnr("$")),
        \ "buflisted(v:val) && (getbufvar(v:val, '&ft') == 'vim')")
  for bufnr in l:buflist
    let l:nbr_of_sourced += s:source_file(bufname(bufnr))
  endfor
  return l:nbr_of_sourced
endfunction

function! sourcer#source_wins()
  " Sources all windows containing vimscript files.
  " Returns: the number of sourced files.
  let g:sourcer#nbr#loaded#sourced = 0
  let g:sourcer#verbose#output = ""
  let l:nbr_of_sourced = 0
  let l:buflist = filter(range(1, bufnr("$")),
        \ "(bufwinnr(v:val) > 0) && (getbufvar(v:val, '&ft') == 'vim')")
  for bufnr in l:buflist
    let l:nbr_of_sourced += s:source_file(bufname(bufnr))
  endfor
  return l:nbr_of_sourced
endfunction

function! sourcer#source_dirs(...)
  " Sources all vimscript-files in directory/ies given as argument/s.
  " If no argument is provided it will source files in the current working
  " directory. If first argument is "R={nr}" function will recursivly source
  " files in subdirectories, "nr" subdirectories deep.
  " Returns: the number of sourced files.
  let g:sourcer#nbr#loaded#sourced = 0
  let g:sourcer#verbose#output = ""
  let l:recursion_steps = 0
  let l:arglist = copy(a:000)
  let l:nbr_of_sourced = 0
  if empty(l:arglist)
    call add(l:arglist, getcwd())
  elseif l:arglist[0] =~# "^R="
    let l:recursion_steps = str2nr(l:arglist[0][2:])
    call remove(l:arglist, 0)
    if empty(l:arglist)
      call add(l:arglist, getcwd())
    endif
  endif
  for argmt in l:arglist
    let l:nbr_of_sourced = s:source_dir_recursion(l:recursion_steps, argmt)
  endfor
  return l:nbr_of_sourced
endfunction

function! s:source_dir_recursion(curr_step, dir)
  " Recursive function that sources files in a directory and calls itself on
  " found subdirectories.
  " Returns: the number of files sourced.
  let g:sourcer#verbose#output .= "SUBDIR: ".a:dir."\n"
  let l:nbr_of_sourced = 0
  if a:curr_step < 0
    return
  endif
  " Source vim-files in directory
  for vimfile in split(glob(a:dir."/*.vim"))
    let l:nbr_of_sourced += s:source_file(vimfile)
  endfor
  " Recursivly call this function on subdirectories
  for subdir in filter(split(glob(a:dir."/*")), "isdirectory(v:val) > 0")
    let l:nbr_of_sourced += s:source_dir_recursion(a:curr_step - 1, subdir)
  endfor
  return l:nbr_of_sourced
endfunction

function! s:source_file(vimfile)
  " Sources a vimscript-file.
  " Returns: 1 if successful, 0 otherwise.
  if g:sourcer_source_loaded
    let g:sourcer#nbr#loaded#sourced += s:unlet_loaded(a:vimfile)
  endif
  try
    execute "source" a:vimfile
    let g:sourcer#verbose#output .= "--SOURCED: ".a:vimfile."\n"
    return 1
  catch /.*/
  let g:sourcer#verbose#output .= "--FAILED: ".a:vimfile."\n"
  endtry
  return 0
endfunction

function! s:unlet_loaded(plugin_file)
  " Unlets loaded variable for a plugin so that it will be sourced even though
  " it has been loaded by vim.
  " Returns: 1 if variable is unlet, 0 otherwise.
  let l:loaded_varname = "g:loaded_".fnamemodify(a:plugin_file, ":t:r")
  if exists(l:loaded_varname) && fnamemodify(a:plugin_file, ":p:h") =~ "plugin$"
    execute "unlet" l:loaded_varname
    let g:sourcer#verbose#output .= "**UNLET: ".l:loaded_varname."\n"
    return 1
  else
    return 0
  endif
endfunction

function! sourcer#output()
  " Provides proper print-output for vim-sourcer's commands.
  " Returns: a string of verbose output.
  if exists("g:sourcer_verbose_output") && g:sourcer_verbose_output == 1
    let l:verbose_output = g:sourcer#verbose#output
    unlet g:sourcer#verbose#output
    return " files sourced\n".l:verbose_output
  endif
  if exists("g:sourcer#nbr#loaded#sourced")
    let l:sourced_loaded = g:sourcer#nbr#loaded#sourced
    unlet g:sourcer#nbr#loaded#sourced
    if l:sourced_loaded > 0
      return "(".l:sourced_loaded.") files sourced"
    else
      return " files sourced"
    endif
  else
    return " files sourced"
  endif
endfunction
